@@ -21,17 +21,19 @@ ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 
 NOTE:
-	xdvi is based on prior work, as noted in the modification history
-	in xdvi.c.
+xdvi is based on prior work, as noted in the modification history
+in xdvi.c.
 
 \*========================================================================*/
 
 #include "xdvi-config.h"
 #include "xdvi.h"
+
 #include "dvi-init.h"
 #include "dvi-draw.h"
 #include "util.h"
 #include "x_util.h"
+#include "exit-handlers.h"
 #include "mime.h"
 #include "pagesel.h"
 #include "special.h"
@@ -132,6 +134,21 @@ get_dvi_error(dviErrFlagT flag) {
     return dvi_err_list[flag];
 }
 
+
+/*
+ * Extract the unit used in paper size specification.  This information is used
+ * to decide the initial grid separation.
+ */
+static int
+atopixunit(const char *arg)
+{
+    int len = strlen(arg);
+
+    /* check if the unit 'mm' or 'cm' occurs in the arg string */
+    return (len > 2 && (arg[len - 2] == 'c' || arg[len - 2] == 'm') && arg[len - 1] == 'm' ?
+	    1.0 / 2.54 : 1.0) * resource.pixels_per_inch + 0.5;
+}
+
 /*
  *	free_vf_chain frees the vf_chain structure.
  */
@@ -221,7 +238,7 @@ free_unused_fonts(void)
 
 		    for (m = fontp->macro;
 			 m <= fontp->macro + fontp->maxchar; ++m)
-			    if (m->free_me) free((char *)m->pos);
+			if (m->free_me) free((char *)m->pos);
 		    free((char *)fontp->macro);
 		    free((char *)fontp->vf_table);
 		    free_vf_chain(fontp->vf_chain);
@@ -325,8 +342,8 @@ realloc_font(struct font *fontp, wide_ubyte newsize)
     glyph = fontp->glyph = xrealloc(fontp->glyph,
 				    (unsigned int)(newsize + 1) * sizeof(struct glyph));
     if (newsize > fontp->maxchar)
-	bzero((char *)(glyph + fontp->maxchar + 1),
-	      (int)(newsize - fontp->maxchar) * sizeof(struct glyph));
+	memset((char *)(glyph + fontp->maxchar + 1), 0,
+	       (int)(newsize - fontp->maxchar) * sizeof(struct glyph));
     maxchar = fontp->maxchar = newsize;
 }
 
@@ -343,8 +360,8 @@ realloc_virtual_font(struct font *fontp, wide_ubyte newsize)
     macro = fontp->macro = xrealloc(fontp->macro,
 				    (unsigned int)(newsize + 1) * sizeof(struct macro));
     if (newsize > fontp->maxchar)
-	bzero((char *)(macro + fontp->maxchar + 1),
-	      (int)(newsize - fontp->maxchar) * sizeof(struct macro));
+	memset((char *)(macro + fontp->maxchar + 1), 0,
+	       (int)(newsize - fontp->maxchar) * sizeof(struct macro));
     maxchar = fontp->maxchar = newsize;
 }
 
@@ -391,11 +408,11 @@ load_font(struct font *fontp, Boolean use_t1lib)
 #ifdef T1LIB
     fontp->file = font_open(fontp->fontname, &font_found,
 			    fsize, &size_found,
-			    &fontp->filename, &fontp->t1id);
+			    &fontp->filename, &fontp->t1id, use_t1lib);
 #else
     fontp->file = font_open(fontp->fontname, &font_found,
 			    fsize, &size_found,
-			    &fontp->filename);
+			    &fontp->filename, use_t1lib);
 #endif /* T1LIB */
 
 #ifdef T1LIB
@@ -408,7 +425,7 @@ load_font(struct font *fontp, Boolean use_t1lib)
 	/* read_T1_char is a dummy */
 	fontp->read_char = read_T1_char;
 	fontp->glyph = xmalloc (256 * sizeof (struct glyph));
-	bzero ((char *) fontp->glyph, 256 * sizeof (struct glyph));
+	memset((char *) fontp->glyph, 0, 256 * sizeof (struct glyph));
 	fontp->flags |= FONT_LOADED;
 	if (font_found != NULL) {
 	    statusline_print(STATUS_MEDIUM,
@@ -488,8 +505,9 @@ load_font(struct font *fontp, Boolean use_t1lib)
     else {
 	while (maxchar > 0 && fontp->glyph[maxchar].addr == 0)
 	    --maxchar;
-	if (maxchar < 255)
+	if (maxchar < 255) {
 	    realloc_font(fontp, (wide_ubyte)maxchar);
+	}
     }
     return True;
 }
@@ -811,6 +829,8 @@ set_paper_type(const char *arg)
     m_paper_unshrunk_w = atopix(arg, False);
     m_paper_unshrunk_h = atopix(arg1 + 1, False);
 
+    globals.grid_paper_unit = atopixunit(arg);
+    
     return (m_paper_unshrunk_w != 0 && m_paper_unshrunk_h != 0);
 }
 
@@ -833,7 +853,7 @@ read_postamble(FILE *fp, dviErrFlagT *errflag, Boolean load_fonts)
     TRACE_FILES((stderr, "read_postamble: reading %p (%d)", (void *)fp, load_fonts));
 
     /* clear existing font table */
-    bzero((char *)tn_table, (int)sizeof tn_table);
+    memset((char *)tn_table, 0, (int)sizeof tn_table);
     free_vf_chain(tn_head);
     tn_head = NULL;
     for (fontp = font_head; fontp != NULL; fontp = fontp->next)
@@ -966,9 +986,10 @@ char *get_tmp_dvi_name() {
     return tmp_dvi_name;
 }
 
-void
-remove_tmp_dvi_file(void)
+static void
+remove_tmp_dvi_file(void *dummy)
 {
+    UNUSED(dummy);
     if (tmp_dvi_name != NULL)
 	unlink(tmp_dvi_name);
 }
@@ -999,7 +1020,7 @@ make_backup_fp(FILE *source_fp, FILE *target_fp)
 	    XDVI_ERROR((stderr, "error creating temporary file - disabling `useTempFp'."));
 	    return NULL;
 	}
-/* 	fprintf(stderr, "temporary file name: |%s|, %d\n", tmp_dvi_name, tmp_fd); */
+	/* 	fprintf(stderr, "temporary file name: |%s|, %d\n", tmp_dvi_name, tmp_fd); */
 	TRACE_EVENTS((stderr, "Created temp file: |%s|\n", tmp_dvi_name));
 	if ((target_fp = try_fdopen(tmp_fd, "wb+")) == NULL) {
 	    XDVI_ERROR((stderr, "error opening temporary file (%s) - disabling `useTempFp'.", strerror(errno)));
@@ -1051,12 +1072,12 @@ file_exists_p(const char *path, dviErrFlagT *errflag)
     TRACE_FILES((stderr, "file_exists_p for |%s|", path));
     *errflag = UNKNOWN_ERROR;
     if ((m_dvi_fp = XFOPEN(path, OPEN_MODE)) == NULL) {
-/*  	fprintf(stderr, "after internal_open_dvi1: xfopen\n"); */
+	/*  	fprintf(stderr, "after internal_open_dvi1: xfopen\n"); */
 	*errflag = FILE_DOESNT_EXIST;
 	return False;
     }
     TRACE_FILES((stderr, "m_dvi_fp for |%s| = %p", path, (void *)m_dvi_fp));
-/*      fprintf(stderr, "after internal_open_dvi2: xfopen\n"); */
+    /*      fprintf(stderr, "after internal_open_dvi2: xfopen\n"); */
     
     /* shouldn't happen */
     if (fstat(fileno(m_dvi_fp), &fstatbuf) != 0 || S_ISDIR(fstatbuf.st_mode)) {	/* if it's a directory */
@@ -1142,9 +1163,9 @@ internal_init_dvi(dviErrFlagT *errflag, Boolean load_fonts)
 Boolean
 internal_open_dvi(const char *path, dviErrFlagT *errflag, Boolean load_fonts)
 {
-/*     FILE *tmp_fp = NULL; */
-/*     static FILE *bak_fp = NULL; /\* re-use the temporary backup fp *\/ */
-/*      fprintf(stderr, "------------ opening: |%s|\n", path); */
+    /*     FILE *tmp_fp = NULL; */
+    /*     static FILE *bak_fp = NULL; /\* re-use the temporary backup fp *\/ */
+    /*      fprintf(stderr, "------------ opening: |%s|\n", path); */
 
     TRACE_FILES((stderr, "internal_open_dvi for |%s|", path));
     close_old_filep();
@@ -1157,6 +1178,8 @@ internal_open_dvi(const char *path, dviErrFlagT *errflag, Boolean load_fonts)
 	globals.dvi_file.bak_fp = m_dvi_fp;
     }
     globals.dvi_file.time = fstatbuf.st_mtime;
+
+    register_exit_handler(remove_tmp_dvi_file, NULL);
     
     if (!internal_init_dvi(errflag, load_fonts)) {
 	return False;
@@ -1345,7 +1368,7 @@ open_dvi_file_wrapper(const char *filename,
 	  people, who meant to launch *xdvi*). `find_dvi_file' tries to locate the
 	  file and does all error handling; on success, it returns a fully expanded
 	  filename.
-	 */
+	*/
 	real_filename = find_dvi_file(filename, tried_dvi_ext, from_file_history); /* this allocates real_filename */
 	if (real_filename == NULL)
 	    return False;
@@ -1362,7 +1385,7 @@ open_dvi_file_wrapper(const char *filename,
 	  if it's not from the command line (e.g. result of clicking Mouse-1 on a link);
 	  in this case we might fall back to using the browser.
 	  Check whether it's a local file:
-	 */
+	*/
 	const char *filename_no_prefix;
 	char *expanded_filename;
 	TRACE_HTEX((stderr, "filename NOT from commandline"));
@@ -1416,7 +1439,7 @@ open_dvi_file(const char *filename, Boolean open_new_instance)
 	dviErrFlagT errflag = NO_ERROR;
 	TRACE_HTEX((stderr, "internal_open_dvi: |%s|", filename));
 	if (!internal_open_dvi(filename, &errflag, True)) {
-/*  	    || !internal_init_dvi(&errflag, True)) { */
+	    /*  	    || !internal_init_dvi(&errflag, True)) { */
 	    report_open_error("Cannot open DVI file", filename, errflag);
 	    retval = False;
 	}
@@ -1477,7 +1500,7 @@ dvi_file_changed(void)
 {
     TRACE_FILES((stderr, "dvi_file_changed: fp = %p?", (void *)m_dvi_fp));
 
-/*     fprintf(stderr, "m_dvi_fp3: %p (%s)\n", m_dvi_fp, globals.dvi_name); */
+    /*     fprintf(stderr, "m_dvi_fp3: %p (%s)\n", m_dvi_fp, globals.dvi_name); */
     if (m_dvi_fp == NULL) {
 	TRACE_FILES((stderr, "m_dvi_fp == NULL"));
 	if (stat(globals.dvi_name, &fstatbuf) == 0 && fstatbuf.st_mtime != globals.dvi_file.time) {
@@ -1508,9 +1531,9 @@ dvi_file_changed(void)
 	       on the canvas.
 	    */
 	    /*
-	    else {
-		globals.ev.flags |= EV_RELOAD;
-	    }
+	      else {
+	      globals.ev.flags |= EV_RELOAD;
+	      }
 	    */
 	    TRACE_FILES((stderr, "returning TRUE"));
 	    return True;
@@ -1549,7 +1572,7 @@ dvi_file_changed(void)
 	    }
 	    else {
 		TRACE_FILES((stderr, "NO successful load: %s", get_dvi_error(errflag)));
-/*  		fprintf(stderr, "=========== NO successful load: %s\n", get_dvi_error(errflag)); */
+		/*  		fprintf(stderr, "=========== NO successful load: %s\n", get_dvi_error(errflag)); */
 		return False;
 	    }
 	}
@@ -1593,9 +1616,9 @@ load_dvi_file(Boolean load_fonts, dviErrFlagT *errflag)
 
     *errflag = NO_ERROR;
     if (!internal_open_dvi(globals.dvi_name, errflag, load_fonts)) {
-/*  	|| !internal_init_dvi(errflag, load_fonts)) { */
+	/*  	|| !internal_init_dvi(errflag, load_fonts)) { */
 	XClearWindow(DISP, mane.win);
-	XBell(DISP, 0);
+	xdvi_bell();
 	statusline_print(STATUS_MEDIUM, "%s: %s%s", globals.dvi_name, get_dvi_error(*errflag),
 			 resource.watch_file > 0.0 ? "; will try to reload ..." : " (click on window to reload)");
 	close_old_filep();
