@@ -30,6 +30,7 @@
 #include "xm_menu.h"
 #include "x_util.h"
 #include "pagehist.h"
+#include "util.h"
 
 #include "kpathsea/c-vararg.h"
 #include "my-vsnprintf.h"
@@ -71,12 +72,12 @@ static void
 handle_statusline_event(Widget w, XtPointer client_data,
 			XEvent *ev, Boolean *cont)
 {
-/*      const char *text = (const char *)client_data; */
+    /*      const char *text = (const char *)client_data; */
     UNUSED(w);
     UNUSED(client_data);
     UNUSED(cont);
 
-/*      fprintf(stderr, "text: |%s|; event: %p\n", text, ev); */
+    /*      fprintf(stderr, "text: |%s|; event: %p\n", text, ev); */
     /* only used to do this if page history was already active, but it's probably
        nicer to be able to get the history by clicking on the statusline ...
     */
@@ -84,7 +85,7 @@ handle_statusline_event(Widget w, XtPointer client_data,
 	XmTextPosition pos = XmTextGetCursorPosition(statusline);
 	char *ptr1, *ptr2;
 	int diff = 0;
-/*  	fprintf(stderr, "pos: %d\n", pos); */
+	/*  	fprintf(stderr, "pos: %d\n", pos); */
 	if (pos == 0) { /* just display the page history */
 	    page_history_move(0);
 	    return;
@@ -95,7 +96,7 @@ handle_statusline_event(Widget w, XtPointer client_data,
 	    page_history_move(0);
 	    return;
 	}
-/* 	fprintf(stderr, "ptr1: |%s|; ptr2: |%s|\n", ptr1, ptr2); */
+	/* 	fprintf(stderr, "ptr1: |%s|; ptr2: |%s|\n", ptr1, ptr2); */
 	while (ptr1 < ptr2) {
 	    if (*ptr1 == ' ' && *(ptr1 + 1) != '-') /* separator */
 		diff--;
@@ -107,7 +108,7 @@ handle_statusline_event(Widget w, XtPointer client_data,
 		diff++;
 	    ptr1--;
 	}
-/* 	fprintf(stderr, "diff: %d\n", diff); */
+	/* 	fprintf(stderr, "diff: %d\n", diff); */
 	page_history_move(diff);
     }
 }
@@ -167,8 +168,8 @@ create_statusline(
     XtVaGetValues(globals.widgets.clip_widget, XtNx, &clip_x, XtNwidth, &clip_w, NULL);
     XtVaGetValues(globals.widgets.vport_widget, XtNheight, &vport_h, NULL);
     if (vport_h - my_h <= 0) {
-	/* the return value is only here to justify the function declaration */
-	return statusline;
+	XDVI_FATAL((stderr, "Window height too small for statusline (minimum value: %d).", my_h));
+	return NULL;
     }
     statusline = XtVaCreateManagedWidget("statusline",
 					 labelWidgetClass, globals.widgets.vport_widget,
@@ -244,11 +245,7 @@ toggle_statusline(void)
     else
 	XtManageChild(statusline);
 
-#if defined(NEW_MENU_CREATION) || defined(MOTIF)
     set_menu(&resource.expert_mode, Act_set_expert_mode, check_resource_expert);
-#else
-    set_show_statusline_option();
-#endif
 #else
     static Boolean initialized = False;
     static Boolean statusline_mapped = False;
@@ -328,29 +325,29 @@ handle_statusline_resize(void)
 
 #if 0
     /*
-    BROKEN  Position vport_h, clip_x, clip_w;
-    BROKEN  static Position my_h = 0;
-    BROKEN
-    BROKEN  XtVaGetValues(globals.widgets.clip_widget,
-    BROKEN                XtNx, &clip_x,
-    BROKEN                XtNwidth, &clip_w,
-    BROKEN                NULL);
-    BROKEN  XtVaGetValues(globals.widgets.vport_widget,
-    BROKEN                XtNheight, &vport_h,
-    BROKEN                NULL);
-    BROKEN
-    BROKEN  XtUnmanageChild(statusline);
-    BROKEN  XtVaSetValues(statusline,
-    BROKEN                             XtNlabel, (XtArgVal) "",
-    BROKEN                XtNwidth, clip_w,
-    BROKEN                XtNx, clip_x - 1,
-    BROKEN                XtNy, vport_h - my_h,
-    BROKEN                XtNborderWidth, 1,
-    BROKEN                XtNjustify, XtJustifyLeft,
-    BROKEN                XtNborder, (XtArgVal) resource.fore_Pixel,
-    BROKEN                NULL);
-    BROKEN  XtManageChild(statusline);
-    BROKEN  XFlush(DISP);
+      BROKEN  Position vport_h, clip_x, clip_w;
+      BROKEN  static Position my_h = 0;
+      BROKEN
+      BROKEN  XtVaGetValues(globals.widgets.clip_widget,
+      BROKEN                XtNx, &clip_x,
+      BROKEN                XtNwidth, &clip_w,
+      BROKEN                NULL);
+      BROKEN  XtVaGetValues(globals.widgets.vport_widget,
+      BROKEN                XtNheight, &vport_h,
+      BROKEN                NULL);
+      BROKEN
+      BROKEN  XtUnmanageChild(statusline);
+      BROKEN  XtVaSetValues(statusline,
+      BROKEN                             XtNlabel, (XtArgVal) "",
+      BROKEN                XtNwidth, clip_w,
+      BROKEN                XtNx, clip_x - 1,
+      BROKEN                XtNy, vport_h - my_h,
+      BROKEN                XtNborderWidth, 1,
+      BROKEN                XtNjustify, XtJustifyLeft,
+      BROKEN                XtNborder, (XtArgVal) resource.fore_Pixel,
+      BROKEN                NULL);
+      BROKEN  XtManageChild(statusline);
+      BROKEN  XFlush(DISP);
     */
 #endif
 
@@ -462,17 +459,17 @@ internal_print_statusline(statusTimerT timeout, const char *old_content, const c
 #else
 	XtVaSetValues(statusline, XtNlabel, buf, NULL);
 #endif
-/* 	fprintf(stderr, "timeout: %d, id: %ld\n", timeout, clear_timeout_id); */
+	/* 	fprintf(stderr, "timeout: %d, id: %ld\n", timeout, clear_timeout_id); */
 	if (timeout > 0) {
 	    timeout *= 1000;	/* convert to miliseconds */
 
 	    if (clear_timeout_id) {
-/* 		fprintf(stderr, "clearing!\n"); */
+		/* 		fprintf(stderr, "clearing!\n"); */
 		if (globals.debug & DBG_EVENT)
 		    fprintf(stderr, "%s:%d: removing timeout %ld\n", __FILE__, __LINE__, clear_timeout_id);
 		XtRemoveTimeOut(clear_timeout_id);
 	    }
-	    clear_timeout_id = XtAppAddTimeOut(app, timeout,
+	    clear_timeout_id = XtAppAddTimeOut(globals.app, timeout,
 					       clear_statusline_timer_proc, (XtPointer) NULL);
 	}
     }
@@ -498,7 +495,7 @@ statusline_append(statusTimerT timeout, const char *pattern, const char *fmt, ..
 #endif
     }
 
-    while (buf != NULL && isspace(*buf)) /* skip spaces inserted by statusline appending */
+    while (buf != NULL && isspace((int)*buf)) /* skip spaces inserted by statusline appending */
 	buf++;
     va_start(argp, fmt);
 
@@ -528,3 +525,25 @@ statusline_clear(void)
 {
     statusline_print(STATUS_SHORT, "");
 }
+
+/*
+ * Erase the contents of the statusline if it starts with 'pattern'.
+ */
+void
+statusline_erase(const char *pattern)
+{
+    const char *buf = NULL;
+    if (XtIsRealized(globals.widgets.top_level) && initialized && (resource.expert_mode & XPRT_SHOW_STATUSLINE) != 0) {
+	/* get current statusline contents */
+#ifdef MOTIF
+	XtVaGetValues(statusline, XmNvalue, &buf, NULL);
+#else
+	XtVaGetValues(statusline, XtNlabel, &buf, NULL);
+#endif
+
+	if (strncmp(buf, pattern, strlen(pattern)) == 0) {
+	    statusline_clear();
+	}
+    }
+}
+
