@@ -201,19 +201,19 @@ get_window_id(char *window_p)
 {
     Window w;
     
-#ifndef WORD64
+#if !(defined(WORD64) || defined(LONG64))
     w = (*((xuint32 *) window_p));
 #else
 # if WORDS_BIGENDIAN
-    w = ((unsigned long)wp[0] << 24) |
-	((unsigned long)wp[1] << 16) |
-	((unsigned long)wp[2] << 8)  |
-	(unsigned long)wp[3];
+    w = ((unsigned long)window_p[0] << 24) |
+	((unsigned long)window_p[1] << 16) |
+	((unsigned long)window_p[2] << 8)  |
+	(unsigned long)window_p[3];
 # else
-    w = ((unsigned long)wp[3] << 24) |
-	((unsigned long)wp[2] << 16) |
-	((unsigned long)wp[1] << 8)  |
-	(unsigned long)wp[0];
+    w = ((unsigned long)window_p[3] << 24) |
+	((unsigned long)window_p[2] << 16) |
+	((unsigned long)window_p[1] << 8)  |
+	(unsigned long)window_p[0];
 # endif
 #endif
     return w;
@@ -360,7 +360,7 @@ update_window_property(Window w, Boolean prepend)
     }
     
     if (prepend) { /* add our ID again to front */
-#ifdef WORD64
+#if (defined(WORD64) || defined(LONG64))
 	unsigned char data[4];
 	set_window_id(w, data);
 #else
@@ -497,7 +497,7 @@ int xm_get_width(Widget w)
 /*
  * Get pixel from color `colorname'. We try to keep this as
  * high-level as possible, with simple black as fallback.
-*/
+ */
 void
 str_to_pixel(Widget w, const char *colorname, Pixel *ret)
 {
@@ -521,7 +521,7 @@ str_to_pixel(Widget w, const char *colorname, Pixel *ret)
  * Convert pixel value `pix' to color name passed as str,
  * of length len, or `black' if conversion failed.
  * -- FIXME: This is broken!!!
-*/
+ */
 void
 pixel_to_str(Widget w, Pixel pix, char *str, size_t len)
 {
@@ -602,6 +602,35 @@ get_matching_parent(Widget w, Widget d, const char *fmt, ...)
     return parent;
 }
 
+void
+adjust_width_to_max(Widget w, ...)
+{
+    Dimension max = 0;
+    Widget w1 = w;
+    
+    va_list argp;
+    va_start(argp, w);
+
+    /* get maximum width */
+    while (w1 != NULL) {
+	Dimension curr;
+	XtVaGetValues(w1, XtNwidth, &curr, NULL);
+	if (curr > max)
+	    max = curr;
+	w1 = va_arg(argp, Widget);
+    }
+    va_end(argp);
+
+    /* set maximum width */
+    va_start(argp, w);
+    w1 = w;
+    while (w1 != NULL) {
+	XtVaSetValues(w1, XtNwidth, max, NULL);
+	w1 = va_arg(argp, Widget);
+    }
+    va_end(argp);
+}
+
 /* Return True if p is a parent of widget w, stopping (and returning FALSE)
    if s is reached (which should be the toplevel window of the hierarchy).
 */
@@ -742,16 +771,20 @@ block_event_callback(Widget w, XtPointer client_data,
 }
 
 
+/* Get a widget with `name' somewhere in the widget hierarchy below `parent'
+   (matching is done against `*name') and return it in `ret'.
+   If `report_error' is True, a warning message is popped up if the widget isn't found.
+*/
 Boolean
 get_widget_by_name(Widget *ret, Widget parent, const char *name, Boolean report_error)
 {
     char buf[1024];
     Widget test;
 
-/*      if (parent == 0 || !XtIsManaged(parent)) { */
-/*  	fprintf(stderr, "Widget %p not managed!\n", parent); */
-/*  	return False; */
-/*      } */
+    /*      if (parent == 0 || !XtIsManaged(parent)) { */
+    /*  	fprintf(stderr, "Widget %p not managed!\n", parent); */
+    /*  	return False; */
+    /*      } */
     
     if (strlen(name) > 1023) {
 	popup_message(globals.widgets.top_level,
@@ -760,7 +793,10 @@ get_widget_by_name(Widget *ret, Widget parent, const char *name, Boolean report_
 		      "Widget name `%s' too long, couldn't get parent", name);
 	return False;
     }
-    sprintf(buf, "*%s", name);
+
+    buf[0] = '*'; /* add wildcard to also match paths */
+    strcpy(buf + 1, name);
+
     if ((test = XtNameToWidget(parent, buf)) != NULL) {
 	*ret = test;
 	return True;
@@ -826,14 +862,15 @@ merge_into_user_db(XrmDatabase db)
 const char *const xdvirc_filename = ".xdvirc";
 const char *const xdvirc_signature_line = "!!! ~/.xdvirc, used by xdvi(1) to save user preferences.\n";
 const char *const xdvirc_header = ""
-	"!!!\n"
-	"!!! Do not edit this file, it will be overwritten by xdvi.\n"
-	"!!! This file contains resources that have been set via the\n"
-	"!!! menus/dialogs. The contents of this file will override\n"
-	"!!! the entries in your ~/.Xdefaults file (but not the command-line\n"
-	"!!! arguments passed to xdvi). Remove this file if you want\n"
-	"!!! to get rid of all these customizations.\n"
-	"!!!\n";
+"!!!\n"
+"!!! Do not edit this file, it will be overwritten by xdvi.\n"
+"!!! This file contains resources that have been set via the\n"
+"!!! menus/dialogs. The contents of this file will override\n"
+"!!! the entries in your ~/.Xdefaults file (but not the command\n"
+"!!! line arguments passed to xdvi). Remove this file\n"
+"!!! if you want to get rid of all these customizations,\n"
+"!!! or start xdvi with the `-q' option to ignore this file.\n"
+"!!!\n";
 
 static char *
 get_xdvirc_path(const char *basename)
@@ -859,7 +896,7 @@ save_geometry(void)
     int x_off, y_off;
     Dimension w, h;
     Window dummy;
-/*     char *geom_str; */
+    /*     char *geom_str; */
     
     (void)XTranslateCoordinates(DISP, XtWindow(globals.widgets.top_level),
 				RootWindowOfScreen(SCRN),
@@ -870,7 +907,7 @@ save_geometry(void)
 		  XtNwidth, &w,
 		  XtNheight, &h,
 #ifdef MOTIF
-/* 		  XmNgeometry, &geom_str, */
+		  /* 		  XmNgeometry, &geom_str, */
 #endif
 		  NULL);
     TRACE_GUI((stderr, "geometry: %dx%d+%d+%d", w, h, x_off, y_off));
@@ -889,6 +926,7 @@ save_user_preferences(Boolean full_save)
     char *xdvirc_name;
     char *tmpname;
     FILE *from_fp, *to_fp;
+    int fd;
 
     if (resource.no_init_file
 	|| m_user_db == NULL) /* nothing to do */
@@ -926,23 +964,37 @@ save_user_preferences(Boolean full_save)
     tmpname = xstrdup(xdvirc_name);
     tmpname = xstrcat(tmpname, ".tmp");
 
+    /* since XrmPutFileDatabase doesn't give a useful error message if it fails,
+       check that creating the file works beforehand. The file is created with 0600 permissions. */
+    if ((fd = try_open_mode(tmpname, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR)) < 0) {
+	XDVI_ERROR((stderr, "Could not save preferences!\nOpening %s for writing failed: %s", tmpname, strerror(errno)));
+	return True;
+    }
+    close(fd);
+    
     XrmPutFileDatabase(m_user_db, tmpname);
 
     if (full_save) {
 	if ((from_fp = try_fopen(tmpname, "r")) == NULL) {
-	    XDVI_ERROR((stderr, "opening %s for reading failed", tmpname));
+	    XDVI_ERROR((stderr, "Could not save preferences!\nOpening %s for reading failed: %s", tmpname, strerror(errno)));
 	    return True;
 	}
 	
-	if ((to_fp = try_fopen(xdvirc_name, "w+")) == NULL) {
-	    XDVI_ERROR((stderr, "opening %s for writing failed", xdvirc_name));
+	/* again, create the file with 600 permissions */
+	if ((fd = try_open_mode(xdvirc_name, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR)) < 0) {
+	    XDVI_ERROR((stderr, "Could not save preferences!\nOpening %s for writing failed: %s", xdvirc_name, strerror(errno)));
+	    return True;
+	}
+	
+	if ((to_fp = fdopen(fd, "w")) == NULL) {
+	    XDVI_ERROR((stderr, "Could not save preferences!\nfdopen for %s for writing failed: %s", xdvirc_name, strerror(errno)));
 	    return True;
 	}
 	
 	if (fputs(xdvirc_signature_line, to_fp) == EOF
 	    || fputs(xdvirc_header, to_fp) == EOF
 	    || !copy_fp(from_fp, to_fp)) {
-	    XDVI_ERROR((stderr, "error writing to %s: %s", xdvirc_name, strerror(errno)));
+	    XDVI_ERROR((stderr, "Could not save preferences!\nError writing to %s: %s", xdvirc_name, strerror(errno)));
 	}
 
 	fclose(from_fp);
@@ -1060,8 +1112,7 @@ get_xdvi_window_id(Boolean same_file, property_cbT callback)
     Window ret_window = 0;
 
     /*
-     * Get window list.  Copy it over (we'll be calling property_get_data()
-     * again).
+     * Get window list.  Copy it over (we'll be calling property_get_data() again).
      */
     if ((window_list_len = property_get_window_list(&p)) == 0)
 	return 0;
@@ -1096,7 +1147,7 @@ get_xdvi_window_id(Boolean same_file, property_cbT callback)
 	    TRACE_CLIENT((stderr, "Window %08lx: doesn't exist any more, deleting", w));
 	    window_list_len -= 4;
 	    window_list_end -= 4;
-	    bcopy(wp + 4, wp, window_list_end - wp);
+	    memmove(wp, wp + 4, window_list_end - wp);
 	    wp -= 4; /* new item is now at wp; don't skip it in next iteration */
 	    need_rewrite = True;
 	    continue;
@@ -1196,3 +1247,8 @@ clip_region_to_rect(XRectangle *rect)
     return ret;
 }
 
+Boolean window_is_mapped(Window w, Display *dpy)
+{
+    XWindowAttributes xwa;
+    return XGetWindowAttributes(dpy, w, &xwa) && xwa.map_state == IsViewable;
+}
