@@ -54,6 +54,7 @@ in xdvi.c.
 #ifndef EXIT_SUCCESS
 #define EXIT_SUCCESS 0
 #endif
+
 #ifndef EXIT_FAILURE
 #define EXIT_FAILURE 1
 #endif
@@ -67,18 +68,22 @@ in xdvi.c.
 #if HAVE_SYS_WAIT_H
 # include <sys/wait.h>
 #endif
+
 #ifndef WIFEXITED
 # define WIFEXITED(status)	(((status) & 255) == 0)
 #endif
+
 #ifndef WEXITSTATUS
 # define WEXITSTATUS(status)	((unsigned)(status) >> 8)
 #endif
+
 #ifndef WIFSIGNALED
 # ifndef WIFSTOPPED
 #  define WIFSTOPPED(status)	(((status) & 0xff) == 0x7f)
 # endif
 # define WIFSIGNALED(status)	(!WIFSTOPPED(status) && !WIFEXITED(status))
 #endif
+
 #ifndef WTERMSIG
 # define WTERMSIG(status)	((status) & 0x7f)
 #endif
@@ -132,7 +137,7 @@ in xdvi.c.
 #  include <X11/Xlib.h>
 #  define X_NOT_STDC_ENV
 # endif
-#endif
+#endif /* STDC_HEADERS */
 
 /* For wchar_t et al., that the X files might want. */
 #include "kpathsea/systypes.h"
@@ -141,6 +146,7 @@ in xdvi.c.
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h> /* this should define MAXPATHLEN */
 #endif
+
 #include "kpathsea/c-pathmx.h" /* get fallback for PATH_MAX if all else fails */
 
 #include <X11/Xlib.h>	/* include Xfuncs.h, if available */
@@ -158,18 +164,7 @@ in xdvi.c.
 # endif
 #endif
 
-#if defined(CFG2RES) && !defined(SELFAUTO)
-# define SELFAUTO 1
-#endif
-
-#if defined(SELFAUTO) && !defined(DEFAULT_CONFIG_PATH)
-# define DEFAULT_CONFIG_PATH "$SELFAUTODIR:$SELFAUTOPARENT"
-#endif
-
-/* NOTE: we don't use CFGFILE */
-
 typedef	char		Bool3;		/* Yes/No/Maybe */
-
 #define	True	1
 #define	False	0
 #define	Maybe	2
@@ -215,9 +210,11 @@ typedef unsigned long xuint32;
 #ifndef STDIN_FILENO
 #define STDIN_FILENO 0
 #endif
+
 #ifndef STDOUT_FILENO
 #define STDOUT_FILENO 1
 #endif
+
 #ifndef STDERR_FILENO
 #define STDERR_FILENO 2
 #endif
@@ -248,6 +245,7 @@ extern KPSEDLL char *kpathsea_version_string;
 #endif
 
 #include "kpathsea/c-vararg.h"
+#include "kpathsea/c-fopen.h"
 
 #ifndef	_XFUNCPROTOBEGIN
 # define _XFUNCPROTOBEGIN
@@ -277,18 +275,6 @@ extern KPSEDLL char *kpathsea_version_string;
 # endif
 #endif
 
-#ifndef	NORETURN
-# ifdef	__GNUC__
-#  ifndef __STRICT_ANSI__
-#   define	NORETURN	volatile
-#  else
-#   define	NORETURN	/* as nothing */
-#  endif
-# else
-#  define	NORETURN	/* as nothing */
-# endif
-#endif
-
 #ifndef	OPEN_MODE
 /*
  * SU, 2001/01/07: xdvi defines OPEN_MODE as "r" or as "r", "ctx=stm" (for VMS),
@@ -315,13 +301,13 @@ extern KPSEDLL char *kpathsea_version_string;
 
 typedef	unsigned char	ubyte;
 
-#if NeedWidePrototypes
+/* #if NeedWidePrototypes */ /* bug #1570481 */
 typedef	unsigned int	wide_ubyte;
 typedef	int		wide_bool;
-#else
-typedef	ubyte		wide_ubyte;
-typedef	Boolean		wide_bool;
-#endif
+/* #else */
+/* typedef	ubyte		wide_ubyte; */
+/* typedef	Boolean		wide_bool; */
+/* #endif */
 
 #if defined(MAKEPK) && !defined(MKTEXPK)
 # define MKTEXPK 1
@@ -389,6 +375,41 @@ typedef	setcharRetvalT (*set_char_proc) (
 # define PS	0
 #endif
 
+#ifdef MOTIF
+#  include <Xm/Xm.h>
+
+/* Note: non-k xdvi has MOTIF_TIMERS here, we use XDVI_XT_TIMER_HACK instead (see above) */
+
+#  ifndef DDIST
+#    define DDIST 4
+#  endif
+#  ifndef DDIST_MAJOR
+#    define DDIST_MAJOR 10
+#  endif
+#  ifndef DDIST_MINOR
+#    define DDIST_MINOR 5
+#  endif
+
+extern  XmStringCharSet G_charset;
+
+/*
+ * Whether to use some widgets which are only available with Motif 2.0.
+ * In these cases, the LessTif versions are either buggy or visually
+ * inferior (as of lesstif-0.93.18), in which case suitable replacements
+ * should be used; e.g. cascade button or a textfield plus button for
+ * a combobox, or a textfield plus button for a spinbox.
+ */
+#  if XmVersion >= 2000 && !defined(LESSTIF_VERSION)
+#    define USE_COMBOBOX 1
+#    define Xdvi_COMBO_BOX_ARROW_SIZE 15 /* the default arrows are *way* too big */
+#    define USE_SPINBOX 1
+#  else
+#    define USE_COMBOBOX 0
+#    define USE_SPINBOX 0
+#  endif
+
+#endif /* MOTIF */
+
 typedef enum {
     XPRT_SHOW_NONE = 0U,
     XPRT_SHOW_STATUSLINE = 1,
@@ -401,7 +422,7 @@ typedef enum {
 #else
     XPRT_SHOW_BUTTONS = 4,
     XPRT_SHOW_ALL = 7
-#endif
+#endif /* MOTIF */
 } expertFlagT;
 
 typedef enum {
@@ -477,9 +498,11 @@ extern struct x_resources {
     Boolean	list_fonts;
     Boolean	reverse;
     Boolean	warn_spec;
+    Boolean	hush;
     Boolean	hush_chars;
     Boolean	hush_chk;
     Boolean	hush_stdout;
+    Boolean	hush_bell;
     Boolean	safer;
 #ifdef VMS
     const char *fore_color;
@@ -512,6 +535,7 @@ extern struct x_resources {
     Boolean	gs_alpha;
     const char *gs_path;
     const char *gs_palette;
+    int		gs_timeout;
 # endif
 # ifdef	MAGICK
     Boolean	useMAGICK;
@@ -546,7 +570,7 @@ extern struct x_resources {
     int		tooltips_wait_period; /* used for communication with Tip.c */
     int		tooltips_wait_period_bak; /* uncustomized value */
     Boolean	show_tooltips;
-#endif
+#endif /* MOTIF */
     Boolean	pagelist_highlight_current;
     Dimension	pagelist_width;
     const char *mg_arg[5];
@@ -566,6 +590,7 @@ extern struct x_resources {
     const char *dvips_options_str;
     int		default_saving_format;
     int		default_printing_target;
+    int         grid_mode;
     char       *rule_color;
     Pixel	rule_pixel;
     int		link_style;
@@ -574,10 +599,16 @@ extern struct x_resources {
     char       *browser;
 #ifdef MOTIF
     char       *prefs_browser_list;
+#  if USE_COMBOBOX
+    char       *search_history;
+    int		search_history_size;
 #endif
+#endif /* MOTIF */
     char       *unknown_mime_suffix;
     char       *no_mime_suffix;
     char       *anchor_pos;
+    /* whether to open file in new window from file selector (only for DVI files) */
+    Boolean    filesel_open_new_window;
     /* bitmask of current search window settings */
     unsigned int    search_window_defaults;
     /*     char *    _scroll_pages; */
@@ -593,7 +624,7 @@ extern struct x_resources {
     int         page_history_size;
 } resource;
 
-extern void reload_app_resources(void);
+extern void load_app_resources(Boolean also_pixels);
 
 
 struct WindowRec {
@@ -705,8 +736,9 @@ struct dvi_file_info {
  * as a replacement for global variables:
  */
 extern struct program_globals {
+    XtAppContext app;
     const char *program_name;	/* argv[0] without the directory part */
-    const char *xdvi_dir;	/* directory where xdvi is running in (for childs who need to change back to it) */
+    const char *cwd;		/* directory where xdvi is running in (for childs who need to change back to it) */
     char *dvi_name;		/* dvi file name, fully expanded with REALPATH */
     Boolean load_init_file;	/* whether to read/save ~/.xdvirc */
     char *orig_locale;		/* original locale we are running in */
@@ -720,6 +752,9 @@ extern struct program_globals {
        also contains the offset that user has set via Act_declare_page_number() */
     int pageno_correct;
 
+    /* paper unit used for grid mode */
+    unsigned int grid_paper_unit;
+    
     /* whether we warn about unrecognized specials. Copy of resource.warn_spec value,
        set to False in the drawing routine to reduce the amount of warnings given.
        Better maybe replace with hash lookup similar to fonts warnings? */
@@ -767,10 +802,6 @@ extern struct WindowRec mane;
 extern struct WindowRec currwin;
 extern struct WindowRec magnifier;
 
-#ifdef MOTIF
-#include <Xm/Xm.h>
-extern  XmStringCharSet G_charset;
-#endif
 
 extern XtAppContext app;
 extern	char		*dvi_property;		/* for setting in window */
@@ -909,6 +940,8 @@ extern Boolean dvi_file_corrupted;
 
 extern short magnifier_stat;	/* 1 = wait for expose, -1 = destroy upon expose */
 
+extern void run_dvi_file(const char *filename, void *data);
+
 /*
   ================================================================================
   globals from dvi-draw.c
@@ -1003,29 +1036,13 @@ extern Widget statusline;
 # define XDVI_XT_TIMER_HACK 0
 #endif
 
-/* globally used GUI stuff */
-#ifdef MOTIF
-#include <Xm/Xm.h>
-/* Note: non-k xdvi has MOTIF_TIMERS here, we use XDVI_XT_TIMER_HACK instead (see above) */
-
-# ifndef DDIST
-#  define DDIST 4
-# endif
-# ifndef DDIST_MAJOR
-#  define DDIST_MAJOR 10
-# endif
-# ifndef DDIST_MINOR
-#  define DDIST_MINOR 5
-# endif
-#endif
-
 extern const char **get_paper_types(void);
 extern size_t get_paper_types_size(void);
 
-extern size_t get_magglass_items(void);
-extern int get_magglass_width(int idx);
-extern int get_magglass_height(int idx);
-
+struct startup_info {
+    int file_idx;
+    char *page_arg;
+};
 
 #ifdef STATUSLINE
 /* this is only for the initialization; the statusline will reset it to a more adequate value: */
@@ -1053,14 +1070,6 @@ struct geom_info {
     void *geom_data;
 };
 
-#ifdef	CFG2RES
-struct cfg2res {
-    const char *cfgname;	/* name in config file */
-    const char *resname;	/* name of resource */
-    Boolean numeric;	/* if numeric */
-};
-#endif
-
 #define get_byte(fp)	((unsigned char)getc(fp))
 #define get_lbyte(fp)	((long)get_byte(fp))
 
@@ -1108,7 +1117,27 @@ extern void motif_translations_hack(void);
 # define vfork fork
 #endif /* HAVE_GOOD_SETSID_VFORK */
 
-/* enable following if you get unresolved messages about `iconv_open' etc.: */
+
+/* XawtextScrollWhenNeeded has been ditched in Xaw7
+   (e.g. Xfree >= 4.x); from the man page:
+   
+   The value XawtextScrollWhenNeeded (and whenNeeded, recognized by
+   the converter), is accepted for backwards compatibilty with
+   resource specifications written for the Xaw6 Text widget, but
+   ignored (effectively treated as XawtextScrollNever).
+
+   So we're forced to use `scrollAlways' here.
+*/
+#define XAW_SCROLL_ALWAYS XawtextScrollAlways
+
+/* the reconfig stuff has been fixed in XFree 4.1.0, vendor release 6510 (Slackware 8.0) */
+#define BROKEN_RECONFIG ((				\
+	(strstr(ServerVendor(DISP), "XFree") != NULL)	\
+	&& VendorRelease(DISP) >= 4000			\
+	&& VendorRelease(DISP) < 4002			\
+	))
+
+/* enable the following if you're getting unresolved messages about `iconv_open' etc.: */
 #if 0
 #define LIBICONV_PLUG
 #define iconv_open libiconv_open
